@@ -1,7 +1,7 @@
 /*
  *      main.h
  *      
- *      Copyright 2012-2014 Alex <alex@linuxonly.ru>
+ *      Copyright 2012-2017 Alex <alex@linuxonly.ru>
  *      
  *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -22,18 +22,24 @@
 
 #include <gtk/gtk.h>
 
-#include "resources.h"
+#include "../resources.h"
 #include "settings.h"
 #include "modem-settings.h"
 #include "notifications.h"
 #include "ayatana.h"
+#include "providersdb.h"
 #include "addressbooks.h"
 
 #if RESOURCE_SPELLCHECKER_ENABLED
 	#include <gtkspell/gtkspell.h>
 #endif
 
+#if RESOURCE_INDICATOR_ENABLED
+	#include <libappindicator/app-indicator.h>
+#endif
+
 #define MMGUI_MAIN_DEFAULT_DEVICE_IDENTIFIER  "00000000000000000000000"
+#define MMGUI_MAIN_DEFAULT_CONNECTION_UUID    "00000000-0000-4000-0000-000000000000"
 
 #define MMGUI_MAIN_OPERATION_TIMEOUT          120
 
@@ -54,7 +60,8 @@ enum _mmgui_main_infobar_type {
 	MMGUI_MAIN_INFOBAR_TYPE_INFO = 0,
 	MMGUI_MAIN_INFOBAR_TYPE_WARNING,
 	MMGUI_MAIN_INFOBAR_TYPE_ERROR,
-	MMGUI_MAIN_INFOBAR_TYPE_PROGRESS
+	MMGUI_MAIN_INFOBAR_TYPE_PROGRESS,
+	MMGUI_MAIN_INFOBAR_TYPE_PROGRESS_UNSTOPPABLE
 };
 
 /*Infobar results*/
@@ -65,9 +72,12 @@ enum _mmgui_main_infobar_result {
 	MMGUI_MAIN_INFOBAR_RESULT_TIMEOUT
 };
 
+typedef gboolean (*mmgui_infobar_close_func)(gpointer data);
+
 struct _mmgui_main_window {
-	//Window
+	/*Window*/
 	GtkWidget *window;
+	GtkWidget *windowbox;
 	GtkWidget *toolbar;
 	GtkWidget *statusbar;
 	GtkWidget *infobar;
@@ -75,8 +85,11 @@ struct _mmgui_main_window {
 	GtkWidget *infobarimage;
 	GtkWidget *infobarlabel;
 	GtkWidget *infobarstopbutton;
+	mmgui_infobar_close_func infobarcallback;
 	guint infobartimeout;
 	guint sbcontext;
+	guint menuitemcount;
+	gboolean infobarlock;
 	GtkWidget *signalimage;
 	GdkPixbuf *signal0icon;
 	GdkPixbuf *signal25icon;
@@ -84,12 +97,14 @@ struct _mmgui_main_window {
 	GdkPixbuf *signal75icon;
 	GdkPixbuf *signal100icon;
 	GdkPixbuf *mainicon;
-	//Dialogs
+	GdkPixbuf *symbolicicon;
+	/*Dialogs*/
 	GtkWidget *aboutdialog;
 	GtkWidget *prefdialog;
 	GtkWidget *questiondialog;
 	GtkWidget *errordialog;
 	GtkWidget *exitdialog;
+	GtkWidget *pinentrydialog;
 	/*Welcome window*/
 	GtkWidget *welcomewindow;
 	GtkWidget *welcomeimage;
@@ -107,6 +122,8 @@ struct _mmgui_main_window {
 	GtkWidget *newsmssendtb;
 	GtkWidget *newsmssavetb;
 	GtkWidget *newsmsspellchecktb;
+	GtkWidget *newsmscounterlabel;
+	GtkWidget *newsmslanguagelabel;
 	GtkTreeStore *smsnumlistmodel;
 	GtkTreePath *smsnumlistmodempath;
 	GtkTreePath *smsnumlistgnomepath;
@@ -119,7 +136,7 @@ struct _mmgui_main_window {
         GtkWidget *newsmslangmenu;
         GSList *newsmscodes;
     #endif
-	//Toolbar
+	/*Toolbar*/
 	GtkWidget *devbutton;
 	GtkWidget *smsbutton;
 	GtkWidget *ussdbutton;
@@ -127,9 +144,9 @@ struct _mmgui_main_window {
 	GtkWidget *scanbutton;
 	GtkWidget *contactsbutton;
 	GtkWidget *trafficbutton;
-	//Pages
+	/*Pages*/
 	GtkWidget *notebook;
-	//Devices page
+	/*Devices page*/
 	GtkWidget *devlist;
 	GtkWidget *devconnctl;
 	GtkWidget *devconneditor;
@@ -138,26 +155,21 @@ struct _mmgui_main_window {
 	GtkWidget *conneditdialog;
 	GtkWidget *connaddtoolbutton;
 	GtkWidget *connremovetoolbutton;
-	GtkWidget *connsavetoolbutton;
 	GtkWidget *contreeview;
-	GtkWidget *connnamecombobox;
-	GtkWidget *connnamecomboboxentry;
+	GtkWidget *connnameentry;
 	GtkWidget *connnameapnentry;
-	GtkWidget *connnethomeradiobutton;
-	GtkWidget *connnetroamradiobutton;
-	GtkWidget *connnetidentry;
+	GtkWidget *connnetroamingcheckbutton;
+	GtkWidget *connnetidspinbutton;
 	GtkWidget *connauthnumberentry;
 	GtkWidget *connauthusernameentry;
 	GtkWidget *connauthpassentry;
-	GtkWidget *conndnsdynradiobutton;
-	GtkWidget *conndnsstradiobutton;
 	GtkWidget *conndns1entry;
 	GtkWidget *conndns2entry;
-	/*Add connection dialog*/
-	GtkWidget *connadddialog;
-	//SMS page
-	GtkWidget *smsinfobar;
-	GtkWidget *smsinfobarlabel;
+	GtkWidget *providersmenu;
+	/*PIN entry dialog*/
+	GtkWidget *pinentry;
+	GtkWidget *pinentryapplybutton;
+	/*SMS page*/
 	GtkWidget *smslist;
 	GtkWidget *smstext;
 	GtkWidget *newsmsbutton;
@@ -165,12 +177,15 @@ struct _mmgui_main_window {
 	GtkWidget *answersmsbutton;
 	GdkPixbuf *smsreadicon;
 	GdkPixbuf *smsunreadicon;
+	GdkPixbuf *smsrecvfoldericon;
+	GdkPixbuf *smssentfoldericon;
+	GdkPixbuf *smsdraftsfoldericon;
 	GtkTreePath *incomingpath;
 	GtkTreePath *sentpath;
 	GtkTreePath *draftspath;
 	GtkTextTag *smsheadingtag;
 	GtkTextTag *smsdatetag;
-	//Info page
+	/*Info page*/
 	GtkWidget *devicevlabel;
 	GtkWidget *operatorvlabel;
 	GtkWidget *operatorcodevlabel;
@@ -184,9 +199,7 @@ struct _mmgui_main_window {
 	GtkWidget *equipmentimage;
 	GtkWidget *networkimage;
 	GtkWidget *locationimage;
-	//USSD page
-	GtkWidget *ussdinfobar;
-	GtkWidget *ussdinfobarlabel;
+	/*USSD page*/
 	GtkWidget *ussdentry;
 	GtkWidget *ussdcombobox;
 	GtkWidget *ussdeditor;
@@ -197,20 +210,16 @@ struct _mmgui_main_window {
 	GtkTextTag *ussdanswertag;
 	GtkEntryCompletion *ussdcompletion;
     GtkListStore *ussdcompletionmodel;
-	//Scan page
-	GtkWidget *scaninfobar;
-	GtkWidget *scaninfobarlabel;
+	/*Scan page*/
 	GtkWidget *scanlist;
 	GtkWidget *startscanbutton;
 	GtkWidget *scancreateconnectionbutton;
-	//Traffic page
+	/*Traffic page*/
 	GtkWidget *trafficparamslist;
 	GtkWidget *trafficdrawingarea;
 	GtkWidget *trafficlimitbutton;
 	GtkWidget *trafficconnbutton;
-	//Contacts page
-	GtkWidget *contactsinfobar;
-	GtkWidget *contactsinfobarlabel;
+	/*Contacts page*/
 	GtkWidget *newcontactbutton;
 	GtkWidget *removecontactbutton;
 	GtkWidget *smstocontactbutton;
@@ -219,7 +228,7 @@ struct _mmgui_main_window {
 	GtkTreePath *contmodempath;
 	GtkTreePath *contgnomepath;
 	GtkTreePath *contkdepath;
-	//New contact dialog
+	/*New contact dialog*/
 	GtkWidget *newcontactdialog;
 	GtkWidget *contactnameentry;
 	GtkWidget *contactnumberentry;
@@ -228,7 +237,7 @@ struct _mmgui_main_window {
 	GtkWidget *contactname2entry;
 	GtkWidget *contactnumber2entry;
 	GtkWidget *newcontactaddbutton;
-	//Limits dialog
+	/*Limits dialog*/
 	GtkWidget *trafficlimitsdialog;
 	GtkWidget *trafficlimitcheckbutton;
 	GtkWidget *trafficamount;
@@ -240,32 +249,34 @@ struct _mmgui_main_window {
 	GtkWidget *timeunits;
 	GtkWidget *timemessage;
 	GtkWidget *timeaction;
-	//Connections dialog
+	/*Connections dialog*/
 	GtkAccelGroup *connaccelgroup;
 	GtkWidget *conndialog;
 	GtkWidget *connscrolledwindow;
 	GtkWidget *conntreeview;
 	GtkWidget *conntermtoolbutton;
-	//Traffic statistics dialog
+	/*Traffic statistics dialog*/
 	GtkWidget *trafficstatsdialog;
 	GtkWidget *trafficstatstreeview;
 	GtkWidget *trafficstatsmonthcb;
 	GtkWidget *trafficstatsyearcb;
-	//USSD editor dialog
+	/*USSD editor dialog*/
 	GtkAccelGroup *ussdaccelgroup;
 	GtkWidget *ussdeditdialog;
 	GtkWidget *ussdedittreeview;
 	GtkWidget *newussdtoolbutton;
 	GtkWidget *removeussdtoolbutton;
 	GtkWidget *ussdencodingtoolbutton;
-	//Preferences
+	/*Preferences*/
 	GtkWidget *prefsmsconcat;
 	GtkWidget *prefsmsexpand;
 	GtkWidget *prefsmsoldontop;
 	GtkWidget *prefsmsvalidityscale;
 	GtkWidget *prefsmsreportcb;
+	GtkWidget *prefsmscommandentry;
 	GtkWidget *preftrafficrxcolor;
 	GtkWidget *preftraffictxcolor;
+	GtkWidget *preftrafficmovdircombo;
 	GtkWidget *prefbehavioursounds;
 	GtkWidget *prefbehaviourhide;
 	GtkWidget *prefbehaviourgeom;
@@ -276,14 +287,25 @@ struct _mmgui_main_window {
 	GtkWidget *prefscannetworkstimeoutscale;
 	GtkWidget *prefmodulesmmcombo;
 	GtkWidget *prefmodulescmcombo;
-	//Exit dialog
+	GtkWidget *prefactivepagessmscb;
+	GtkWidget *prefactivepagesussdcb;
+	GtkWidget *prefactivepagesinfocb;
+	GtkWidget *prefactivepagesscancb;
+	GtkWidget *prefactivepagestrafficcb;
+	GtkWidget *prefactivepagescontactscb;
+	/*Exit dialog*/
 	GtkWidget *exitaskagain;
 	GtkWidget *exitcloseradio;
 	GtkWidget *exithideradio;
 	/*Keyboard acceletators*/
 	GtkAccelGroup *accelgroup;
+	/*Pages menu section*/
+	GMenu *appsection;
 	/*Page shortcuts*/
 	GSList *pageshortcuts;
+	/*Closures for Devices page*/
+	GClosure *conneditorclosure;
+	GClosure *connactivateclosure;
 	/*Closures for SMS page*/
 	GClosure *newsmsclosure;
 	GClosure *removesmsclosure;
@@ -301,10 +323,10 @@ struct _mmgui_main_window {
 	GClosure *newcontactclosure;
 	GClosure *removecontactclosure;
 	GClosure *smstocontactclosure;
-	/*Tray icon*/
-	GtkStatusIcon *statusicon;
-	GtkWidget *traymenu;
-	GtkWidget *showwin_tm, *sep1_tm, *newsms_tm, *sep2_tm, *quit_tm;
+	/*Indicator*/
+	AppIndicator *indicator;
+	GtkWidget *indmenu;
+	GtkWidget *showwin_ind, *sep1_ind, *newsms_ind, *sep2_ind, *quit_ind;
 	gulong traysigid;
 };
 
@@ -314,15 +336,14 @@ struct _mmgui_cli_options {
 	gboolean invisible;
 	gboolean minimized;
 	gboolean hidenotifyshown;
-	/*gboolean nostatistics;
-	gboolean nosmsupdate;*/
-	//SMS
+	/*SMS*/
 	gboolean concatsms;
 	gboolean smsexpandfolders;
 	gboolean smsoldontop;
 	gboolean smsdeliveryreport;
 	gint smsvalidityperiod;
-	//Traffic graph
+	gchar *smscustomcommand;
+	/*Traffic graph*/
 	#if GTK_CHECK_VERSION(3,4,0)
 		GdkRGBA rxtrafficcolor;
 		GdkRGBA txtrafficcolor;
@@ -330,29 +351,36 @@ struct _mmgui_cli_options {
 		GdkColor rxtrafficcolor;
 		GdkColor txtrafficcolor;
 	#endif
-	//Behaviour
+	gboolean graphrighttoleft;
+	/*Behaviour*/
 	gboolean usesounds;
 	gboolean hidetotray;
 	gboolean askforhide;
 	gboolean savegeometry;
-	//Window geometry
+	/*Window geometry*/
 	gint wgwidth;
 	gint wgheight;
 	gint wgposx;
 	gint wgposy;
+	/*Active pages*/
+	gboolean smspageenabled;
+	gboolean ussdpageenabled;
+	gboolean infopageenabled;
+	gboolean scanpageenabled;
+	gboolean trafficpageenabled;
+	gboolean contactspageenabled;
 };
 
 typedef struct _mmgui_cli_options *mmgui_cli_options_t;
 
 struct _mmgui_application {
-	//GTK+ application object
+	/*GTK+ application object*/
 	GtkApplication *gtkapplication;
-	//Allocated structures
+	/*Allocated structures*/
 	mmgui_main_window_t window;
 	mmgui_cli_options_t options;
-	//mmgui_traffic_limits_t limits;
 	mmgui_core_options_t coreoptions;
-	//Objects
+	/*Objects*/
 	mmguicore_t core;
 	settings_t settings;
 	modem_settings_t modemsettings;
@@ -360,6 +388,7 @@ struct _mmgui_application {
 	mmgui_notifications_t notifications;
 	mmgui_addressbooks_t addressbooks;
 	mmgui_ayatana_t ayatana;
+	mmgui_providers_db_t providersdb;
 };
 
 typedef struct _mmgui_application *mmgui_application_t;
@@ -374,10 +403,8 @@ typedef struct _mmgui_application_data *mmgui_application_data_t;
 
 gboolean mmgui_main_ui_question_dialog_open(mmgui_application_t mmguiapp, gchar *caption, gchar *text);
 gboolean mmgui_main_ui_error_dialog_open(mmgui_application_t mmguiapp, gchar *caption, gchar *text);
-
 void mmgui_ui_infobar_show_result(mmgui_application_t mmguiapp, gint result, gchar *message);
-void mmgui_ui_infobar_show(mmgui_application_t mmguiapp, gchar *message, gint type);
-
+void mmgui_ui_infobar_show(mmgui_application_t mmguiapp, gchar *message, gint type, mmgui_infobar_close_func callback, gchar *buttoncaption);
 gboolean mmgui_main_ui_test_device_state(mmgui_application_t mmguiapp, guint setpage);
 void mmgui_main_ui_devices_button_toggled_signal(GObject *object, gpointer data);
 void mmgui_main_ui_sms_button_toggled_signal(GObject *object, gpointer data);
@@ -386,11 +413,12 @@ void mmgui_main_ui_info_button_toggled_signal(GObject *object, gpointer data);
 void mmgui_main_ui_scan_button_toggled_signal(GObject *object, gpointer data);
 void mmgui_main_ui_traffic_button_toggled_signal(GObject *object, gpointer data);
 void mmgui_main_ui_contacts_button_toggled_signal(GObject *object, gpointer data);
+void mmgui_main_window_update_active_pages(mmgui_application_t mmguiapp);
 gboolean mmgui_main_ui_window_delete_event_signal(GtkWidget *widget, GdkEvent  *event, gpointer data);
 void mmgui_main_ui_window_destroy_signal(GObject *object, gpointer data);
 gchar *mmgui_main_ui_message_validity_scale_value_format(GtkScale *scale, gdouble value, gpointer user_data);
 gchar *mmgui_main_ui_timeout_scale_value_format(GtkScale *scale, gdouble value, gpointer user_data);
-void mmgui_main_ui_control_buttons_disable(mmgui_application_t mmguiapp, gboolean disable);
+void mmgui_main_ui_controls_disable(mmgui_application_t mmguiapp, gboolean disable, gboolean firstpage, gboolean updatestate);
 void mmgui_main_ui_interrupt_operation_button_clicked_signal(GObject *object, gpointer data);
 gboolean mmgui_main_ui_update_statusbar_from_thread(gpointer data);
 
