@@ -1,7 +1,7 @@
 /*
  *      svcmanager.c
  *
- *      Copyright 2015 Alex <alex@linuxonly.ru>
+ *      Copyright 2015-2018 Alex <alex@linuxonly.ru>
  *
  *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -164,7 +164,7 @@ static gboolean mmgui_svcmanager_open_systemd_manager_interface(mmgui_svcmanager
 	
 	/*Test if systemd can be used*/
 	if (!mmgui_polkit_action_needed(svcmanager->polkit, "ru.linuxonly.modem-manager-gui.manage-services", TRUE)) {
-		g_debug("Systemd doesnt work with polkit\n");
+		g_debug("Systemd doesn't work with polkit\n");
 		return FALSE;
 	}
 
@@ -208,10 +208,10 @@ static gboolean mmgui_svcmanager_open_systemd_manager_interface(mmgui_svcmanager
 		g_variant_iter_init(&svciter, svclistv);
 		while (g_variant_iter_loop(&svciter, "(ssssssouso)", &svcname, NULL, &svcloadedstr, &svcactivestr, &svcrunningstr, NULL, NULL, NULL, NULL, NULL)) {
 			/*Only services*/
-			if (g_strrstr(svcname, ".service") != NULL) {
+			if ((g_strrstr(svcname, ".service") != NULL) && (g_str_equal(svcloadedstr, "loaded"))) {
 				service = g_new0(struct _mmgui_svcmanager_service, 1);
 				service->name = g_strdup(svcname);
-				service->loaded = g_str_equal(svcloadedstr, "loaded");
+				service->loaded = TRUE;
 				service->active = g_str_equal(svcactivestr, "active");
 				service->running = g_str_equal(svcrunningstr, "running");
 				service->enabled = FALSE;
@@ -383,7 +383,11 @@ mmgui_svcmanager_t mmgui_svcmanager_open(mmgui_polkit_t polkit, mmgui_svcmanager
 	/*Service manager object for system services activation*/
 	svcmanager = g_new0(struct _mmgui_svcmanager, 1);
 	svcmanager->systemdtech = FALSE;
+	svcmanager->managerproxy = NULL;
+	svcmanager->services = NULL;
 	svcmanager->dbustech = FALSE;
+	svcmanager->dbusproxy = NULL;
+	svcmanager->interfaces = NULL;
 	svcmanager->polkit = polkit;
 	svcmanager->callback = callback;
 	svcmanager->userdata = userdata;
@@ -760,8 +764,8 @@ static void mmgui_svcmanager_services_activation_interface_handler(GDBusProxy *p
 			mmgui_svcmanager_services_activation_chain(svcmanager);
 		} else {
 			/*Status is unknown*/
-			g_debug("Interface \'%s\' for module \'%s\' not activated. Unknown actvation status: %u\n", transition->entity.interface->name, transition->modname, status);
-			mmgui_svcmanager_set_last_error(svcmanager, _("Unknown actvation status"));
+			g_debug("Interface \'%s\' for module \'%s\' not activated. Unknown activation status: %u\n", transition->entity.interface->name, transition->modname, status);
+			mmgui_svcmanager_set_last_error(svcmanager, _("Unknown activation status"));
 			if (svcmanager->callback != NULL) {
 				(svcmanager->callback)(svcmanager, MMGUI_SVCMANGER_EVENT_ENTITY_ERROR, transition, svcmanager->userdata);
 			}
@@ -927,8 +931,8 @@ gboolean mmgui_svcmanager_start_services_activation(mmgui_svcmanager_t svcmanage
 	mmgui_svcmanager_services_activation_chain(svcmanager);
 	
 	if (svcmanager->svcsinqueue) {
-		/*Drop autorization*/
-		mmgui_polkit_revoke_autorization(svcmanager->polkit, "ru.linuxonly.modem-manager-gui.manage-services");
+		/*Drop authorization*/
+		mmgui_polkit_revoke_authorization(svcmanager->polkit, "ru.linuxonly.modem-manager-gui.manage-services");
 	}
 	
 	return TRUE;
