@@ -1,7 +1,7 @@
 /*
  *      ussd-page.c
  *      
- *      Copyright 2012-2014 Alex <alex@linuxonly.ru>
+ *      Copyright 2012-2017 Alex <alex@linuxonly.ru>
  *      
  *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 #include "modem-settings.h"
 #include "ussdlist.h"
 #include "mmguicore.h"
-#include "resources.h"
+#include "../resources.h"
 
 #include "ussd-page.h"
 #include "main.h"
@@ -407,7 +407,7 @@ void mmgui_main_ussd_request_send(mmgui_application_t mmguiapp)
 				gtk_text_buffer_delete_mark(buffer, position);
 				//show progress dialog
 				//mmgui_main_ui_progress_dialog_open(mmguiapp);
-				mmgui_ui_infobar_show(mmguiapp, statusmsg, MMGUI_MAIN_INFOBAR_TYPE_PROGRESS);
+				mmgui_ui_infobar_show(mmguiapp, statusmsg, MMGUI_MAIN_INFOBAR_TYPE_PROGRESS, NULL, NULL);
 				g_free(statusmsg);
 			} else {
 				mmgui_main_ui_error_dialog_open(mmguiapp, _("<b>Error sending USSD</b>"), _("Wrong USSD request or device not ready"));
